@@ -10,28 +10,33 @@ from duckdb import StarExpression
 from narwhals._duckdb.utils import (
     DeferredTimeZone,
     F,
+    catch_duckdb_exception,
     col,
     evaluate_exprs,
+    join_column_names,
     lit,
     native_to_narwhals_dtype,
     window_expression,
 )
+from narwhals._sql.dataframe import SQLLazyFrame
 from narwhals._utils import (
     Implementation,
+    ValidateBackendVersion,
     Version,
     generate_temporary_column_name,
     not_implemented,
     parse_columns_to_drop,
-    parse_version,
     requires,
-    validate_backend_version,
+    to_pyarrow_table,
+    zip_strict,
 )
 from narwhals.dependencies import get_duckdb
 from narwhals.exceptions import InvalidOperationError
-from narwhals.typing import CompliantLazyFrame
 
 if TYPE_CHECKING:
-    from collections.abc import Iterator, Mapping, Sequence
+    from collections.abc import Iterable, Iterator, Mapping, Sequence
+    from io import BytesIO
+    from pathlib import Path
     from types import ModuleType
 
     import pandas as pd
@@ -45,19 +50,21 @@ if TYPE_CHECKING:
     from narwhals._duckdb.group_by import DuckDBGroupBy
     from narwhals._duckdb.namespace import DuckDBNamespace
     from narwhals._duckdb.series import DuckDBInterchangeSeries
-    from narwhals._utils import _FullContext
+    from narwhals._typing import _EagerAllowedImpl
+    from narwhals._utils import _LimitedContext
     from narwhals.dataframe import LazyFrame
     from narwhals.dtypes import DType
     from narwhals.stable.v1 import DataFrame as DataFrameV1
-    from narwhals.typing import AsofJoinStrategy, JoinStrategy, LazyUniqueKeepStrategy
+    from narwhals.typing import AsofJoinStrategy, JoinStrategy, UniqueKeepStrategy
 
 
 class DuckDBLazyFrame(
-    CompliantLazyFrame[
+    SQLLazyFrame[
         "DuckDBExpr",
         "duckdb.DuckDBPyRelation",
         "LazyFrame[duckdb.DuckDBPyRelation] | DataFrameV1[duckdb.DuckDBPyRelation]",
-    ]
+    ],
+    ValidateBackendVersion,
 ):
     _implementation = Implementation.DUCKDB
 
@@ -65,15 +72,19 @@ class DuckDBLazyFrame(
         self,
         df: duckdb.DuckDBPyRelation,
         *,
-        backend_version: tuple[int, ...],
         version: Version,
+        validate_backend_version: bool = False,
     ) -> None:
         self._native_frame: duckdb.DuckDBPyRelation = df
         self._version = version
-        self._backend_version = backend_version
         self._cached_native_schema: dict[str, DuckDBPyType] | None = None
         self._cached_columns: list[str] | None = None
-        validate_backend_version(self._implementation, self._backend_version)
+        if validate_backend_version:
+            self._validate_backend_version()
+
+    @property
+    def _backend_version(self) -> tuple[int, ...]:
+        return self._implementation._backend_version()
 
     @staticmethod
     def _is_native(obj: duckdb.DuckDBPyRelation | Any) -> TypeIs[duckdb.DuckDBPyRelation]:
@@ -81,21 +92,18 @@ class DuckDBLazyFrame(
 
     @classmethod
     def from_native(
-        cls, data: duckdb.DuckDBPyRelation, /, *, context: _FullContext
+        cls, data: duckdb.DuckDBPyRelation, /, *, context: _LimitedContext
     ) -> Self:
-        return cls(
-            data, backend_version=context._backend_version, version=context._version
-        )
+        return cls(data, version=context._version)
 
     def to_narwhals(
         self, *args: Any, **kwds: Any
     ) -> LazyFrame[duckdb.DuckDBPyRelation] | DataFrameV1[duckdb.DuckDBPyRelation]:
-        if self._version is Version.MAIN:
-            return self._version.lazyframe(self, level="lazy")
+        if self._version is Version.V1:
+            from narwhals.stable.v1 import DataFrame as DataFrameV1
 
-        from narwhals.stable.v1 import DataFrame as DataFrameV1
-
-        return DataFrameV1(self, level="interchange")  # type: ignore[no-any-return]
+            return DataFrameV1(self, level="interchange")  # type: ignore[no-any-return]
+        return self._version.lazyframe(self, level="lazy")
 
     def __narwhals_dataframe__(self) -> Self:  # pragma: no cover
         # Keep around for backcompat.
@@ -113,9 +121,7 @@ class DuckDBLazyFrame(
     def __narwhals_namespace__(self) -> DuckDBNamespace:
         from narwhals._duckdb.namespace import DuckDBNamespace
 
-        return DuckDBNamespace(
-            backend_version=self._backend_version, version=self._version
-        )
+        return DuckDBNamespace(version=self._version)
 
     def get_column(self, name: str) -> DuckDBInterchangeSeries:
         from narwhals._duckdb.series import DuckDBInterchangeSeries
@@ -127,40 +133,34 @@ class DuckDBLazyFrame(
             yield col(name)
 
     def collect(
-        self, backend: ModuleType | Implementation | str | None, **kwargs: Any
+        self, backend: _EagerAllowedImpl | None, **kwargs: Any
     ) -> CompliantDataFrameAny:
         if backend is None or backend is Implementation.PYARROW:
-            import pyarrow as pa  # ignore-banned-import
-
             from narwhals._arrow.dataframe import ArrowDataFrame
 
             return ArrowDataFrame(
-                self.native.arrow(),
-                backend_version=parse_version(pa),
+                to_pyarrow_table(self.native.arrow()),
+                validate_backend_version=True,
                 version=self._version,
                 validate_column_names=True,
             )
 
         if backend is Implementation.PANDAS:
-            import pandas as pd  # ignore-banned-import
-
             from narwhals._pandas_like.dataframe import PandasLikeDataFrame
 
             return PandasLikeDataFrame(
                 self.native.df(),
                 implementation=Implementation.PANDAS,
-                backend_version=parse_version(pd),
+                validate_backend_version=True,
                 version=self._version,
                 validate_column_names=True,
             )
 
         if backend is Implementation.POLARS:
-            import polars as pl  # ignore-banned-import
-
             from narwhals._polars.dataframe import PolarsDataFrame
 
             return PolarsDataFrame(
-                self.native.pl(), backend_version=parse_version(pl), version=self._version
+                self.native.pl(), validate_backend_version=True, version=self._version
             )
 
         msg = f"Unsupported `backend` value: {backend}"  # pragma: no cover
@@ -174,18 +174,24 @@ class DuckDBLazyFrame(
 
     def aggregate(self, *exprs: DuckDBExpr) -> Self:
         selection = [val.alias(name) for name, val in evaluate_exprs(self, *exprs)]
-        return self._with_native(self.native.aggregate(selection))  # type: ignore[arg-type]
+        try:
+            return self._with_native(self.native.aggregate(selection))  # type: ignore[arg-type]
+        except Exception as e:  # noqa: BLE001
+            raise catch_duckdb_exception(e, self) from None
 
     def select(self, *exprs: DuckDBExpr) -> Self:
         selection = (val.alias(name) for name, val in evaluate_exprs(self, *exprs))
-        return self._with_native(self.native.select(*selection))
+        try:
+            return self._with_native(self.native.select(*selection))
+        except Exception as e:  # noqa: BLE001
+            raise catch_duckdb_exception(e, self) from None
 
     def drop(self, columns: Sequence[str], *, strict: bool) -> Self:
         columns_to_drop = parse_columns_to_drop(self, columns, strict=strict)
-        selection = (name for name in self.columns if name not in columns_to_drop)
+        selection = [col(name) for name in self.columns if name not in columns_to_drop]
         return self._with_native(self.native.select(*selection))
 
-    def lazy(self, *, backend: Implementation | None = None) -> Self:
+    def lazy(self, backend: None = None, **_: None) -> Self:
         # The `backend`` argument has no effect but we keep it here for
         # backwards compatibility because in `narwhals.stable.v1`
         # function `.from_native()` will return a DataFrame for DuckDB.
@@ -204,12 +210,18 @@ class DuckDBLazyFrame(
             for name in self.columns
         ]
         result.extend(value.alias(name) for name, value in new_columns_map.items())
-        return self._with_native(self.native.select(*result))
+        try:
+            return self._with_native(self.native.select(*result))
+        except Exception as e:  # noqa: BLE001
+            raise catch_duckdb_exception(e, self) from None
 
     def filter(self, predicate: DuckDBExpr) -> Self:
         # `[0]` is safe as the predicate's expression only returns a single column
         mask = predicate(self)[0]
-        return self._with_native(self.native.filter(mask))
+        try:
+            return self._with_native(self.native.filter(mask))
+        except Exception as e:  # noqa: BLE001
+            raise catch_duckdb_exception(e, self) from None
 
     @property
     def schema(self) -> dict[str, DType]:
@@ -223,7 +235,9 @@ class DuckDBLazyFrame(
             column_name: native_to_narwhals_dtype(
                 duckdb_dtype, self._version, deferred_time_zone
             )
-            for column_name, duckdb_dtype in zip(self.native.columns, self.native.types)
+            for column_name, duckdb_dtype in zip_strict(
+                self.native.columns, self.native.types
+            )
         }
 
     @property
@@ -238,27 +252,17 @@ class DuckDBLazyFrame(
 
     def to_pandas(self) -> pd.DataFrame:
         # only if version is v1, keep around for backcompat
-        import pandas as pd  # ignore-banned-import()
-
-        if parse_version(pd) >= (1, 0, 0):
-            return self.native.df()
-        else:  # pragma: no cover
-            msg = f"Conversion to pandas requires 'pandas>=1.0.0', found {pd.__version__}"
-            raise NotImplementedError(msg)
+        return self.native.df()
 
     def to_arrow(self) -> pa.Table:
         # only if version is v1, keep around for backcompat
-        return self.native.arrow()
+        return self.lazy().collect(Implementation.PYARROW).native  # type: ignore[no-any-return]
 
     def _with_version(self, version: Version) -> Self:
-        return self.__class__(
-            self.native, version=version, backend_version=self._backend_version
-        )
+        return self.__class__(self.native, version=version)
 
     def _with_native(self, df: duckdb.DuckDBPyRelation) -> Self:
-        return self.__class__(
-            df, backend_version=self._backend_version, version=self._version
-        )
+        return self.__class__(df, version=self._version)
 
     def group_by(
         self, keys: Sequence[str] | Sequence[DuckDBExpr], *, drop_null_keys: bool
@@ -297,7 +301,7 @@ class DuckDBLazyFrame(
             assert right_on is not None  # noqa: S101
             it = (
                 col(f'lhs."{left}"') == col(f'rhs."{right}"')
-                for left, right in zip(left_on, right_on)
+                for left, right in zip_strict(left_on, right_on)
             )
             condition: Expression = reduce(and_, it)
             rel = self.native.set_alias("lhs").join(
@@ -342,7 +346,7 @@ class DuckDBLazyFrame(
         if by_left is not None and by_right is not None:
             conditions.extend(
                 col(f'lhs."{left}"') == col(f'rhs."{right}"')
-                for left, right in zip(by_left, by_right)
+                for left, right in zip_strict(by_left, by_right)
             )
         else:
             by_left = by_right = []
@@ -376,25 +380,43 @@ class DuckDBLazyFrame(
         return self.schema
 
     def unique(
-        self, subset: Sequence[str] | None, *, keep: LazyUniqueKeepStrategy
+        self,
+        subset: Sequence[str] | None,
+        *,
+        keep: UniqueKeepStrategy,
+        order_by: Sequence[str] | None,
     ) -> Self:
-        if subset_ := subset if keep == "any" else (subset or self.columns):
-            # Sanitise input
-            if error := self._check_columns_exist(subset_):
-                raise error
-            idx_name = generate_temporary_column_name(8, self.columns)
-            count_name = generate_temporary_column_name(8, [*self.columns, idx_name])
-            name = count_name if keep == "none" else idx_name
-            idx_expr = window_expression(F("row_number"), subset_).alias(idx_name)
-            count_expr = window_expression(
-                F("count", StarExpression()), subset_, ()
-            ).alias(count_name)
-            return self._with_native(
-                self.native.select(StarExpression(), idx_expr, count_expr)
-                .filter(col(name) == lit(1))
-                .select(StarExpression(exclude=[count_name, idx_name]))
+        subset_ = subset or self.columns
+        if error := self._check_columns_exist(subset_):
+            raise error
+        tmp_name = generate_temporary_column_name(8, self.columns)
+        if order_by and keep == "last":
+            descending = [True] * len(order_by)
+            nulls_last = [True] * len(order_by)
+        else:
+            descending = None
+            nulls_last = None
+        if keep == "none":
+            expr = window_expression(
+                F("count", StarExpression()),
+                subset_,
+                order_by or (),
+                descending=descending,
+                nulls_last=nulls_last,
             )
-        return self._with_native(self.native.unique(", ".join(self.columns)))
+        else:
+            expr = window_expression(
+                F("row_number"),
+                subset_,
+                order_by or (),
+                descending=descending,
+                nulls_last=nulls_last,
+            )
+        return self._with_native(
+            self.native.select(StarExpression(), expr.alias(tmp_name)).filter(
+                col(tmp_name) == lit(1)
+            )
+        ).drop([tmp_name], strict=False)
 
     def sort(self, *by: str, descending: bool | Sequence[bool], nulls_last: bool) -> Self:
         if isinstance(descending, bool):
@@ -402,15 +424,36 @@ class DuckDBLazyFrame(
         if nulls_last:
             it = (
                 col(name).nulls_last() if not desc else col(name).desc().nulls_last()
-                for name, desc in zip(by, descending)
+                for name, desc in zip_strict(by, descending)
             )
         else:
             it = (
                 col(name).nulls_first() if not desc else col(name).desc().nulls_first()
-                for name, desc in zip(by, descending)
+                for name, desc in zip_strict(by, descending)
             )
         return self._with_native(self.native.sort(*it))
 
+    def top_k(self, k: int, *, by: Iterable[str], reverse: bool | Sequence[bool]) -> Self:
+        _df = self.native
+        by = list(by)
+        if isinstance(reverse, bool):
+            descending = [not reverse] * len(by)
+        else:
+            descending = [not rev for rev in reverse]
+        expr = window_expression(
+            F("row_number"),
+            order_by=by,
+            descending=descending,
+            nulls_last=[True] * len(by),
+        )
+        condition = expr <= lit(k)
+        query = f"""
+        SELECT *
+        FROM _df
+        QUALIFY {condition}
+        """  # noqa: S608
+        return self._with_native(duckdb.sql(query))
+
     def drop_nulls(self, subset: Sequence[str] | None) -> Self:
         subset_ = subset if subset is not None else self.columns
         keep_condition = reduce(and_, (col(name).isnotnull() for name in subset_))
@@ -476,7 +519,7 @@ class DuckDBLazyFrame(
             msg = "`value_name` cannot be empty string for duckdb backend."
             raise NotImplementedError(msg)
 
-        unpivot_on = ", ".join(str(col(name)) for name in on_)
+        unpivot_on = join_column_names(*on_)
         rel = self.native  # noqa: F841
         # Replace with Python API once
         # https://github.com/duckdb/duckdb/discussions/16980 is addressed.
@@ -484,8 +527,8 @@ class DuckDBLazyFrame(
             unpivot rel
             on {unpivot_on}
             into
-                name "{variable_name}"
-                value "{value_name}"
+                name {col(variable_name)}
+                value {col(value_name)}
             """
         return self._with_native(
             duckdb.sql(query).select(*[*index_, variable_name, value_name])
@@ -493,11 +536,23 @@ class DuckDBLazyFrame(
 
     @requires.backend_version((1, 3))
     def with_row_index(self, name: str, order_by: Sequence[str]) -> Self:
+        if order_by is None:
+            msg = "Cannot pass `order_by` to `with_row_index` for DuckDB"
+            raise TypeError(msg)
         expr = (window_expression(F("row_number"), order_by=order_by) - lit(1)).alias(
             name
         )
         return self._with_native(self.native.select(expr, StarExpression()))
 
+    def sink_parquet(self, file: str | Path | BytesIO) -> None:
+        df = self.native  # noqa: F841
+        query = f"""
+            COPY (SELECT * FROM df)
+            TO '{file}'
+            (FORMAT parquet)
+            """  # noqa: S608
+        duckdb.sql(query)
+
     gather_every = not_implemented.deprecated(
         "`LazyFrame.gather_every` is deprecated and will be removed in a future version."
     )
