@@ -10,25 +10,26 @@ from narwhals._compliant import EagerDataFrame
 from narwhals._pandas_like.series import PANDAS_TO_NUMPY_DTYPE_MISSING, PandasLikeSeries
 from narwhals._pandas_like.utils import (
     align_and_extract_native,
-    align_series_full_broadcast,
-    check_column_names_are_unique,
     get_dtype_backend,
+    import_array_module,
+    iter_dtype_backends,
     native_to_narwhals_dtype,
     object_native_to_narwhals_dtype,
     rename,
     select_columns_by_name,
     set_index,
 )
+from narwhals._typing_compat import assert_never
 from narwhals._utils import (
     Implementation,
     _into_arrow_table,
     _remap_full_join_keys,
+    check_column_names_are_unique,
     exclude_column_names,
     generate_temporary_column_name,
     parse_columns_to_drop,
-    parse_version,
     scale_bytes,
-    validate_backend_version,
+    zip_strict,
 )
 from narwhals.dependencies import is_pandas_like_dataframe
 from narwhals.exceptions import InvalidOperationError, ShapeError
@@ -46,13 +47,15 @@ if TYPE_CHECKING:
     from narwhals._pandas_like.expr import PandasLikeExpr
     from narwhals._pandas_like.group_by import PandasLikeGroupBy
     from narwhals._pandas_like.namespace import PandasLikeNamespace
+    from narwhals._spark_like.utils import SparkSession
     from narwhals._translate import IntoArrowTable
-    from narwhals._utils import Version, _FullContext
+    from narwhals._typing import _EagerAllowedImpl, _LazyAllowedImpl
+    from narwhals._utils import Version, _LimitedContext
     from narwhals.dtypes import DType
-    from narwhals.schema import Schema
     from narwhals.typing import (
         AsofJoinStrategy,
         DTypeBackend,
+        IntoSchema,
         JoinStrategy,
         PivotAgg,
         SizedMultiIndexSelector,
@@ -93,35 +96,42 @@ CLASSICAL_NUMPY_DTYPES: frozenset[np.dtype[Any]] = frozenset(
 )
 
 
-class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "Any"]):
+class PandasLikeDataFrame(
+    EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "Any", "pd.Series[Any]"]
+):
     def __init__(
         self,
         native_dataframe: Any,
         *,
         implementation: Implementation,
-        backend_version: tuple[int, ...],
         version: Version,
         validate_column_names: bool,
+        validate_backend_version: bool = False,
     ) -> None:
         self._native_frame = native_dataframe
         self._implementation = implementation
-        self._backend_version = backend_version
         self._version = version
-        validate_backend_version(self._implementation, self._backend_version)
         if validate_column_names:
             check_column_names_are_unique(native_dataframe.columns)
+        if validate_backend_version:
+            self._validate_backend_version()
 
     @classmethod
-    def from_arrow(cls, data: IntoArrowTable, /, *, context: _FullContext) -> Self:
+    def from_arrow(cls, data: IntoArrowTable, /, *, context: _LimitedContext) -> Self:
         implementation = context._implementation
         tbl = _into_arrow_table(data, context)
         if implementation.is_pandas():
             native = tbl.to_pandas()
-        elif implementation.is_modin():  # pragma: no cover
-            from modin.pandas.utils import (
-                from_arrow as mpd_from_arrow,  # pyright: ignore[reportAttributeAccessIssue]
-            )
-
+        elif implementation.is_modin():
+            # NOTE: Function moved + deprecated (0.26.0), then old path removed (0.31.0)
+            # https://github.com/modin-project/modin/pull/6806
+            # https://github.com/modin-project/modin/pull/7274
+            if implementation._backend_version() >= (0, 26, 0):
+                from modin.pandas.io import from_arrow as mpd_from_arrow
+            else:  # pragma: no cover
+                from modin.pandas.utils import (
+                    from_arrow as mpd_from_arrow,  # pyright: ignore[reportAttributeAccessIssue]
+                )
             native = mpd_from_arrow(tbl)
         elif implementation.is_cudf():  # pragma: no cover
             native = implementation.to_native_namespace().DataFrame.from_arrow(tbl)
@@ -136,15 +146,15 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
         data: Mapping[str, Any],
         /,
         *,
-        context: _FullContext,
-        schema: Mapping[str, DType] | Schema | None,
+        context: _LimitedContext,
+        schema: IntoSchema | None,
     ) -> Self:
         from narwhals.schema import Schema
 
         implementation = context._implementation
         ns = implementation.to_native_namespace()
-        Series = cast("type[pd.Series[Any]]", ns.Series)  # noqa: N806
-        DataFrame = cast("type[pd.DataFrame]", ns.DataFrame)  # noqa: N806
+        Series = cast("type[pd.Series[Any]]", ns.Series)
+        DataFrame = cast("type[pd.DataFrame]", ns.DataFrame)
         aligned_data: dict[str, pd.Series[Any] | Any] = {}
         left_most: PandasLikeSeries | None = None
         for name, series in data.items():
@@ -157,13 +167,15 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
                     aligned_data[name] = align_and_extract_native(left_most, compliant)[1]
             else:
                 aligned_data[name] = series
-
-        native = DataFrame.from_dict(aligned_data)
+        if aligned_data or not schema:
+            native = DataFrame.from_dict(aligned_data)
+        else:
+            native = DataFrame.from_dict({col: [] for col in schema})
         if schema:
-            it: Iterable[DTypeBackend] = (
-                get_dtype_backend(dtype, implementation) for dtype in native.dtypes
-            )
-            native = native.astype(Schema(schema).to_pandas(it))
+            backend: Iterable[DTypeBackend] | None = None
+            if aligned_data:
+                backend = iter_dtype_backends(native.dtypes, implementation)
+            native = native.astype(Schema(schema).to_pandas(backend))
         return cls.from_native(native, context=context)
 
     @staticmethod
@@ -171,11 +183,10 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
         return is_pandas_like_dataframe(obj)  # pragma: no cover
 
     @classmethod
-    def from_native(cls, data: Any, /, *, context: _FullContext) -> Self:
+    def from_native(cls, data: Any, /, *, context: _LimitedContext) -> Self:
         return cls(
             data,
             implementation=context._implementation,
-            backend_version=context._backend_version,
             version=context._version,
             validate_column_names=True,
         )
@@ -186,13 +197,13 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
         data: _2DArray,
         /,
         *,
-        context: _FullContext,
-        schema: Mapping[str, DType] | Schema | Sequence[str] | None,
+        context: _LimitedContext,
+        schema: IntoSchema | Sequence[str] | None,
     ) -> Self:
         from narwhals.schema import Schema
 
         implementation = context._implementation
-        DataFrame: Constructor = implementation.to_native_namespace().DataFrame  # noqa: N806
+        DataFrame: Constructor = implementation.to_native_namespace().DataFrame
         if isinstance(schema, (Mapping, Schema)):
             it: Iterable[DTypeBackend] = (
                 get_dtype_backend(native_type, implementation)
@@ -214,9 +225,7 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
     def __narwhals_namespace__(self) -> PandasLikeNamespace:
         from narwhals._pandas_like.namespace import PandasLikeNamespace
 
-        return PandasLikeNamespace(
-            self._implementation, self._backend_version, version=self._version
-        )
+        return PandasLikeNamespace(self._implementation, version=self._version)
 
     def __native_namespace__(self) -> ModuleType:
         if self._implementation in {
@@ -236,7 +245,6 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
         return self.__class__(
             self.native,
             implementation=self._implementation,
-            backend_version=self._backend_version,
             version=version,
             validate_column_names=False,
         )
@@ -245,7 +253,6 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
         return self.__class__(
             df,
             implementation=self._implementation,
-            backend_version=self._backend_version,
             version=self._version,
             validate_column_names=validate_column_names,
         )
@@ -259,14 +266,17 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
             msg = f"Expected object of length {len_idx}, got: {len_other}."
             raise ShapeError(msg)
         if other.native.index is not index:
-            return set_index(
-                other.native,
-                index,
-                implementation=other._implementation,
-                backend_version=other._backend_version,
-            )
+            return set_index(other.native, index, implementation=other._implementation)
         return other.native
 
+    @property
+    def _array_funcs(self):  # type: ignore[no-untyped-def] # noqa: ANN202
+        if TYPE_CHECKING:
+            import numpy as np
+
+            return np
+        return import_array_module(self._implementation)
+
     def get_column(self, name: str) -> PandasLikeSeries:
         return PandasLikeSeries.from_native(self.native[name], context=self)
 
@@ -312,9 +322,7 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
             self.native.iloc[:, columns], validate_column_names=False
         )
 
-    def _select_multi_name(
-        self, columns: SizedMultiNameSelector[pd.Series[Any]]
-    ) -> PandasLikeDataFrame:
+    def _select_multi_name(self, columns: SizedMultiNameSelector[pd.Series[Any]]) -> Self:
         return self._with_native(self.native.loc[:, columns])
 
     # --- properties ---
@@ -380,55 +388,55 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
     # --- reshape ---
     def simple_select(self, *column_names: str) -> Self:
         return self._with_native(
-            select_columns_by_name(
-                self.native,
-                list(column_names),
-                self._backend_version,
-                self._implementation,
-            ),
+            select_columns_by_name(self.native, list(column_names), self._implementation),
             validate_column_names=False,
         )
 
-    def select(self: PandasLikeDataFrame, *exprs: PandasLikeExpr) -> PandasLikeDataFrame:
+    def select(self, *exprs: PandasLikeExpr) -> Self:
         new_series = self._evaluate_into_exprs(*exprs)
         if not new_series:
             # return empty dataframe, like Polars does
-            return self._with_native(self.native.__class__(), validate_column_names=False)
-        new_series = align_series_full_broadcast(*new_series)
+            return self._with_native(type(self.native)(), validate_column_names=False)
+        new_series = new_series[0]._align_full_broadcast(*new_series)
         namespace = self.__narwhals_namespace__()
         df = namespace._concat_horizontal([s.native for s in new_series])
         # `concat` creates a new object, so fine to modify `.columns.name` inplace.
         df.columns.name = self.native.columns.name
         return self._with_native(df, validate_column_names=True)
 
-    def drop_nulls(
-        self: PandasLikeDataFrame, subset: Sequence[str] | None
-    ) -> PandasLikeDataFrame:
+    def drop_nulls(self, subset: Sequence[str] | None) -> Self:
         if subset is None:
             return self._with_native(
                 self.native.dropna(axis=0), validate_column_names=False
             )
         plx = self.__narwhals_namespace__()
-        return self.filter(~plx.any_horizontal(plx.col(*subset).is_null()))
+        mask = ~plx.any_horizontal(plx.col(*subset).is_null(), ignore_nulls=True)
+        return self.filter(mask)
 
     def estimated_size(self, unit: SizeUnit) -> int | float:
         sz = self.native.memory_usage(deep=True).sum()
         return scale_bytes(sz, unit=unit)
 
-    def with_row_index(self, name: str) -> Self:
-        frame = self.native
-        namespace = self.__narwhals_namespace__()
-        row_index = namespace._series.from_iterable(
-            range(len(frame)), context=self, index=frame.index
-        ).alias(name)
-        return self._with_native(namespace._concat_horizontal([row_index.native, frame]))
+    def with_row_index(self, name: str, order_by: Sequence[str] | None) -> Self:
+        plx = self.__narwhals_namespace__()
+        if order_by is None:
+            size = len(self)
+            data = self._array_funcs.arange(size)
+
+            row_index = plx._expr._from_series(
+                plx._series.from_iterable(
+                    data, context=self, index=self.native.index, name=name
+                )
+            )
+        else:
+            rank = plx.col(order_by[0]).rank(method="ordinal", descending=False)
+            row_index = (rank.over(partition_by=[], order_by=order_by) - 1).alias(name)
+        return self.select(row_index, plx.all())
 
     def row(self, index: int) -> tuple[Any, ...]:
         return tuple(x for x in self.native.iloc[index])
 
-    def filter(
-        self: PandasLikeDataFrame, predicate: PandasLikeExpr | list[bool]
-    ) -> PandasLikeDataFrame:
+    def filter(self, predicate: PandasLikeExpr | list[bool]) -> Self:
         if isinstance(predicate, list):
             mask_native: pd.Series[Any] | list[bool] = predicate
         else:
@@ -439,9 +447,7 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
             self.native.loc[mask_native], validate_column_names=False
         )
 
-    def with_columns(
-        self: PandasLikeDataFrame, *exprs: PandasLikeExpr
-    ) -> PandasLikeDataFrame:
+    def with_columns(self, *exprs: PandasLikeExpr) -> Self:
         columns = self._evaluate_into_exprs(*exprs)
         if not columns and len(self) == 0:
             return self
@@ -463,12 +469,7 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
 
     def rename(self, mapping: Mapping[str, str]) -> Self:
         return self._with_native(
-            rename(
-                self.native,
-                columns=mapping,
-                implementation=self._implementation,
-                backend_version=self._backend_version,
-            )
+            rename(self.native, columns=mapping, implementation=self._implementation)
         )
 
     def drop(self, columns: Sequence[str], *, strict: bool) -> Self:
@@ -490,51 +491,55 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
             validate_column_names=False,
         )
 
+    def top_k(self, k: int, *, by: Iterable[str], reverse: bool | Sequence[bool]) -> Self:
+        df = self.native
+        schema = self.schema
+        if isinstance(reverse, bool) and all(schema[x].is_numeric() for x in by):
+            if reverse:
+                return self._with_native(df.nsmallest(k, by))
+            return self._with_native(df.nlargest(k, by))
+        return self._with_native(
+            df.sort_values(list(by), ascending=reverse).head(k),
+            validate_column_names=False,
+        )
+
     # --- convert ---
     def collect(
-        self, backend: Implementation | None, **kwargs: Any
+        self, backend: _EagerAllowedImpl | None, **kwargs: Any
     ) -> CompliantDataFrameAny:
         if backend is None:
             return PandasLikeDataFrame(
                 self.native,
                 implementation=self._implementation,
-                backend_version=self._backend_version,
                 version=self._version,
                 validate_column_names=False,
             )
 
         if backend is Implementation.PANDAS:
-            import pandas as pd  # ignore-banned-import
-
-            return PandasLikeDataFrame(
-                self.to_pandas(),
-                implementation=Implementation.PANDAS,
-                backend_version=parse_version(pd),
-                version=self._version,
-                validate_column_names=False,
-            )
+            kwds: dict[str, Any] = {
+                "implementation": Implementation.PANDAS,
+                "version": self._version,
+                "validate_column_names": False,
+            }
+            if backend is not self._implementation:
+                kwds.update(validate_backend_version=True)
+            return PandasLikeDataFrame(self.to_pandas(), **kwds)
 
         if backend is Implementation.PYARROW:
-            import pyarrow as pa  # ignore-banned-import
-
             from narwhals._arrow.dataframe import ArrowDataFrame
 
             return ArrowDataFrame(
                 native_dataframe=self.to_arrow(),
-                backend_version=parse_version(pa),
+                validate_backend_version=True,
                 version=self._version,
                 validate_column_names=False,
             )
 
         if backend is Implementation.POLARS:
-            import polars as pl  # ignore-banned-import
-
             from narwhals._polars.dataframe import PolarsDataFrame
 
             return PolarsDataFrame(
-                df=self.to_polars(),
-                backend_version=parse_version(pl),
-                version=self._version,
+                df=self.to_polars(), validate_backend_version=True, version=self._version
             )
 
         msg = f"Unsupported `backend` value: {backend}"  # pragma: no cover
@@ -548,7 +553,140 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
 
         return PandasLikeGroupBy(self, keys, drop_null_keys=drop_null_keys)
 
-    def join(  # noqa: C901, PLR0911, PLR0912
+    def _join_inner(
+        self, other: Self, *, left_on: Sequence[str], right_on: Sequence[str], suffix: str
+    ) -> pd.DataFrame:
+        return self.native.merge(
+            other.native,
+            left_on=left_on,
+            right_on=right_on,
+            how="inner",
+            suffixes=("", suffix),
+        )
+
+    def _join_left(
+        self, other: Self, *, left_on: Sequence[str], right_on: Sequence[str], suffix: str
+    ) -> pd.DataFrame:
+        result_native = self.native.merge(
+            other.native,
+            how="left",
+            left_on=left_on,
+            right_on=right_on,
+            suffixes=("", suffix),
+        )
+        extra = [
+            right_key if right_key not in self.columns else f"{right_key}{suffix}"
+            for left_key, right_key in zip_strict(left_on, right_on)
+            if right_key != left_key
+        ]
+        # NOTE: Keep `inplace=True` to avoid making a redundant copy.
+        # This may need updating, depending on https://github.com/pandas-dev/pandas/pull/51466/files
+        result_native.drop(columns=extra, inplace=True)  # noqa: PD002
+        return result_native
+
+    def _join_full(
+        self, other: Self, *, left_on: Sequence[str], right_on: Sequence[str], suffix: str
+    ) -> pd.DataFrame:
+        # Pandas coalesces keys in full joins unless there's no collision
+        right_on_mapper = _remap_full_join_keys(left_on, right_on, suffix)
+        other_native = other.native.rename(columns=right_on_mapper)
+        check_column_names_are_unique(other_native.columns)
+        right_suffixed = list(right_on_mapper.values())
+        return self.native.merge(
+            other_native,
+            left_on=left_on,
+            right_on=right_suffixed,
+            how="outer",
+            suffixes=("", suffix),
+        )
+
+    def _join_cross(self, other: Self, *, suffix: str) -> pd.DataFrame:
+        implementation = self._implementation
+        backend_version = self._backend_version
+        if (implementation.is_modin() or implementation.is_cudf()) or (
+            implementation.is_pandas() and backend_version < (1, 4)
+        ):
+            key_token = generate_temporary_column_name(
+                n_bytes=8, columns=(*self.columns, *other.columns)
+            )
+            result_native = self.native.assign(**{key_token: 0}).merge(
+                other.native.assign(**{key_token: 0}),
+                how="inner",
+                left_on=key_token,
+                right_on=key_token,
+                suffixes=("", suffix),
+            )
+            # NOTE: Keep `inplace=True` to avoid making a redundant copy.
+            # This may need updating, depending on https://github.com/pandas-dev/pandas/pull/51466/files
+            result_native.drop(columns=key_token, inplace=True)  # noqa: PD002
+            return result_native
+        return self.native.merge(other.native, how="cross", suffixes=("", suffix))
+
+    def _join_semi(
+        self, other: Self, *, left_on: Sequence[str], right_on: Sequence[str]
+    ) -> pd.DataFrame:
+        other_native = self._join_filter_rename(
+            other=other,
+            columns_to_select=list(right_on),
+            columns_mapping=dict(zip(right_on, left_on)),
+        )
+        return self.native.merge(
+            other_native, how="inner", left_on=left_on, right_on=left_on
+        )
+
+    def _join_anti(
+        self, other: Self, *, left_on: Sequence[str], right_on: Sequence[str]
+    ) -> pd.DataFrame:
+        implementation = self._implementation
+
+        if implementation.is_cudf():
+            return self.native.merge(
+                other.native, how="leftanti", left_on=left_on, right_on=right_on
+            )
+
+        indicator_token = generate_temporary_column_name(
+            n_bytes=8, columns=(*self.columns, *other.columns)
+        )
+
+        other_native = self._join_filter_rename(
+            other=other,
+            columns_to_select=list(right_on),
+            columns_mapping=dict(zip(right_on, left_on)),
+        )
+        result_native = self.native.merge(
+            other_native,
+            # TODO(FBruzzesi): See https://github.com/modin-project/modin/issues/7384
+            how="left" if implementation.is_pandas() else "outer",
+            indicator=indicator_token,
+            left_on=left_on,
+            right_on=left_on,
+        ).loc[lambda t: t[indicator_token] == "left_only"]
+        # NOTE: Keep `inplace=True` to avoid making a redundant copy.
+        # This may need updating, depending on https://github.com/pandas-dev/pandas/pull/51466/files
+        result_native.drop(columns=indicator_token, inplace=True)  # noqa: PD002
+        return result_native
+
+    def _join_filter_rename(
+        self, other: Self, columns_to_select: list[str], columns_mapping: dict[str, str]
+    ) -> pd.DataFrame:
+        """Helper function to avoid creating extra columns and row duplication.
+
+        Used in `"anti"` and `"semi`" join's.
+
+        Notice that a native object is returned.
+        """
+        implementation = self._implementation
+        return rename(
+            select_columns_by_name(
+                other.native,
+                column_names=columns_to_select,
+                implementation=implementation,
+            ),
+            columns=columns_mapping,
+            implementation=implementation,
+        ).drop_duplicates()
+
+    def join(
         self,
         other: Self,
         *,
@@ -558,142 +696,31 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
         suffix: str,
     ) -> Self:
         if how == "cross":
-            if (
-                self._implementation is Implementation.MODIN
-                or self._implementation is Implementation.CUDF
-            ) or (
-                self._implementation is Implementation.PANDAS
-                and self._backend_version < (1, 4)
-            ):
-                key_token = generate_temporary_column_name(
-                    n_bytes=8, columns=[*self.columns, *other.columns]
-                )
+            result = self._join_cross(other=other, suffix=suffix)
 
-                return self._with_native(
-                    self.native.assign(**{key_token: 0})
-                    .merge(
-                        other.native.assign(**{key_token: 0}),
-                        how="inner",
-                        left_on=key_token,
-                        right_on=key_token,
-                        suffixes=("", suffix),
-                    )
-                    .drop(columns=key_token)
-                )
-            else:
-                return self._with_native(
-                    self.native.merge(other.native, how="cross", suffixes=("", suffix))
-                )
+        elif left_on is None or right_on is None:  # pragma: no cover
+            raise ValueError(left_on, right_on)
 
-        if how == "anti":
-            if self._implementation is Implementation.CUDF:
-                return self._with_native(
-                    self.native.merge(
-                        other.native, how="leftanti", left_on=left_on, right_on=right_on
-                    )
-                )
-            else:
-                indicator_token = generate_temporary_column_name(
-                    n_bytes=8, columns=[*self.columns, *other.columns]
-                )
-                if right_on is None:  # pragma: no cover
-                    msg = "`right_on` cannot be `None` in anti-join"
-                    raise TypeError(msg)
-
-                # rename to avoid creating extra columns in join
-                other_native = rename(
-                    select_columns_by_name(
-                        other.native,
-                        list(right_on),
-                        self._backend_version,
-                        self._implementation,
-                    ),
-                    columns=dict(zip(right_on, left_on)),  # type: ignore[arg-type]
-                    implementation=self._implementation,
-                    backend_version=self._backend_version,
-                ).drop_duplicates()
-                return self._with_native(
-                    self.native.merge(
-                        other_native,
-                        how="outer",
-                        indicator=indicator_token,
-                        left_on=left_on,
-                        right_on=left_on,
-                    )
-                    .loc[lambda t: t[indicator_token] == "left_only"]
-                    .drop(columns=indicator_token)
-                )
-
-        if how == "semi":
-            if right_on is None:  # pragma: no cover
-                msg = "`right_on` cannot be `None` in semi-join"
-                raise TypeError(msg)
-            # rename to avoid creating extra columns in join
-            other_native = (
-                rename(
-                    select_columns_by_name(
-                        other.native,
-                        list(right_on),
-                        self._backend_version,
-                        self._implementation,
-                    ),
-                    columns=dict(zip(right_on, left_on)),  # type: ignore[arg-type]
-                    implementation=self._implementation,
-                    backend_version=self._backend_version,
-                ).drop_duplicates()  # avoids potential rows duplication from inner join
+        elif how == "inner":
+            result = self._join_inner(
+                other=other, left_on=left_on, right_on=right_on, suffix=suffix
             )
-            return self._with_native(
-                self.native.merge(
-                    other_native, how="inner", left_on=left_on, right_on=left_on
-                )
+        elif how == "anti":
+            result = self._join_anti(other=other, left_on=left_on, right_on=right_on)
+        elif how == "semi":
+            result = self._join_semi(other=other, left_on=left_on, right_on=right_on)
+        elif how == "left":
+            result = self._join_left(
+                other=other, left_on=left_on, right_on=right_on, suffix=suffix
             )
-
-        if how == "left":
-            result_native = self.native.merge(
-                other.native,
-                how="left",
-                left_on=left_on,
-                right_on=right_on,
-                suffixes=("", suffix),
+        elif how == "full":
+            result = self._join_full(
+                other=other, left_on=left_on, right_on=right_on, suffix=suffix
             )
-            extra = []
-            for left_key, right_key in zip(left_on, right_on):  # type: ignore[arg-type]
-                if right_key != left_key and right_key not in self.columns:
-                    extra.append(right_key)
-                elif right_key != left_key:
-                    extra.append(f"{right_key}{suffix}")
-            return self._with_native(result_native.drop(columns=extra))
+        else:
+            assert_never(how)
 
-        if how == "full":
-            # Pandas coalesces keys in full joins unless there's no collision
-
-            # help mypy
-            assert left_on is not None  # noqa: S101
-            assert right_on is not None  # noqa: S101
-
-            right_on_mapper = _remap_full_join_keys(left_on, right_on, suffix)
-            other_native = other.native.rename(columns=right_on_mapper)
-            check_column_names_are_unique(other_native.columns)
-            right_on = list(right_on_mapper.values())  # we now have the suffixed keys
-            return self._with_native(
-                self.native.merge(
-                    other_native,
-                    left_on=left_on,
-                    right_on=right_on,
-                    how="outer",
-                    suffixes=("", suffix),
-                )
-            )
-
-        return self._with_native(
-            self.native.merge(
-                other.native,
-                left_on=left_on,
-                right_on=right_on,
-                how=how,
-                suffixes=("", suffix),
-            )
-        )
+        return self._with_native(result)
 
     def join_asof(
         self,
@@ -734,55 +761,95 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
         *,
         keep: UniqueKeepStrategy,
         maintain_order: bool | None = None,
+        order_by: Sequence[str] | None,
     ) -> Self:
         # The param `maintain_order` is only here for compatibility with the Polars API
         # and has no effect on the output.
         mapped_keep = {"none": False, "any": "first"}.get(keep, keep)
         if subset and (error := self._check_columns_exist(subset)):
             raise error
-        return self._with_native(
-            self.native.drop_duplicates(subset=subset, keep=mapped_keep),
-            validate_column_names=False,
-        )
+        if order_by and maintain_order:
+            token = generate_temporary_column_name(8, self.columns)
+            res = (
+                self.with_row_index(token, order_by=None)
+                .sort(*order_by, nulls_last=False, descending=False)
+                .native.drop_duplicates(subset or self.columns, keep=mapped_keep)
+                .sort_values(token)
+            )
+            res.drop(columns=token, inplace=True)  # noqa: PD002
+        elif order_by:
+            res = self.sort(
+                *order_by, nulls_last=False, descending=False
+            ).native.drop_duplicates(subset or self.columns, keep=mapped_keep)
+        else:
+            res = self.native.drop_duplicates(subset or self.columns, keep=mapped_keep)
+        return self._with_native(res, validate_column_names=False)
 
     # --- lazy-only ---
-    def lazy(self, *, backend: Implementation | None = None) -> CompliantLazyFrameAny:
-        from narwhals.utils import parse_version
-
+    def lazy(
+        self,
+        backend: _LazyAllowedImpl | None = None,
+        *,
+        session: SparkSession | None = None,
+    ) -> CompliantLazyFrameAny:
         pandas_df = self.to_pandas()
         if backend is None:
             return self
-        elif backend is Implementation.DUCKDB:
+        if backend is Implementation.DUCKDB:
             import duckdb  # ignore-banned-import
 
             from narwhals._duckdb.dataframe import DuckDBLazyFrame
 
             return DuckDBLazyFrame(
                 df=duckdb.table("pandas_df"),
-                backend_version=parse_version(duckdb),
+                validate_backend_version=True,
                 version=self._version,
             )
-        elif backend is Implementation.POLARS:
+        if backend is Implementation.POLARS:
             import polars as pl  # ignore-banned-import
 
             from narwhals._polars.dataframe import PolarsLazyFrame
 
             return PolarsLazyFrame(
                 df=pl.from_pandas(pandas_df).lazy(),
-                backend_version=parse_version(pl),
+                validate_backend_version=True,
                 version=self._version,
             )
-        elif backend is Implementation.DASK:
-            import dask  # ignore-banned-import
+        if backend is Implementation.DASK:
             import dask.dataframe as dd  # ignore-banned-import
 
             from narwhals._dask.dataframe import DaskLazyFrame
 
             return DaskLazyFrame(
                 native_dataframe=dd.from_pandas(pandas_df),
-                backend_version=parse_version(dask),
+                validate_backend_version=True,
                 version=self._version,
             )
+        if backend is Implementation.IBIS:
+            import ibis  # ignore-banned-import
+
+            from narwhals._ibis.dataframe import IbisLazyFrame
+
+            return IbisLazyFrame(
+                ibis.memtable(pandas_df, columns=self.columns),
+                validate_backend_version=True,
+                version=self._version,
+            )
+
+        if backend.is_spark_like():
+            from narwhals._spark_like.dataframe import SparkLikeLazyFrame
+
+            if session is None:
+                msg = "Spark like backends require `session` to be not None."
+                raise ValueError(msg)
+
+            return SparkLikeLazyFrame(
+                session.createDataFrame(pandas_df),
+                version=self._version,
+                implementation=backend,
+                validate_backend_version=True,
+            )
+
         raise AssertionError  # pragma: no cover
 
     @property
@@ -835,8 +902,6 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
         # returns Object) then we just call `to_numpy()` on the DataFrame.
         for col_dtype in native_dtypes:
             if str(col_dtype) in PANDAS_TO_NUMPY_DTYPE_MISSING:
-                import numpy as np
-
                 arr: Any = np.hstack(
                     [
                         self.get_column(col).to_numpy(copy=copy, dtype=None)[:, None]
@@ -849,9 +914,9 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
     def to_pandas(self) -> pd.DataFrame:
         if self._implementation is Implementation.PANDAS:
             return self.native
-        elif self._implementation is Implementation.CUDF:
+        if self._implementation is Implementation.CUDF:
             return self.native.to_pandas()
-        elif self._implementation is Implementation.MODIN:
+        if self._implementation is Implementation.MODIN:
             return self.native._to_pandas()
         msg = f"Unknown implementation: {self._implementation}"  # pragma: no cover
         raise AssertionError(msg)
@@ -890,7 +955,7 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
                 raise ValueError(msg)
             return self.native.iloc[0, 0]
 
-        elif row is None or column is None:
+        if row is None or column is None:
             msg = "cannot call `.item()` with only one of `row` or `column`"
             raise ValueError(msg)
 
@@ -971,17 +1036,9 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
         ],
         /,
     ) -> Any:
-        categorical = self._version.dtypes.Categorical
-        kwds: dict[Any, Any] = {"observed": True}
-        if self._implementation is Implementation.CUDF:
-            kwds.pop("observed")
-            cols = set(chain(values, index, on))
-            schema = self.schema.items()
-            if any(
-                tp for name, tp in schema if name in cols and isinstance(tp, categorical)
-            ):
-                msg = "`pivot` with Categoricals is not implemented for cuDF backend"
-                raise NotImplementedError(msg)
+        kwds: dict[Any, Any] = (
+            {} if self._implementation is Implementation.CUDF else {"observed": True}
+        )
         return self.native.pivot_table(
             values=values,
             index=index,
@@ -1001,7 +1058,7 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
     ) -> pd.DataFrame:
         if aggregate_function is None:
             return self.native.pivot(columns=on, index=index, values=values)
-        elif aggregate_function == "len":
+        if aggregate_function == "len":
             return (
                 self.native.groupby([*on, *index], as_index=False)
                 .agg(dict.fromkeys(values, "size"))
@@ -1106,29 +1163,26 @@ class PandasLikeDataFrame(EagerDataFrame["PandasLikeSeries", "PandasLikeExpr", "
             return self._with_native(
                 self.native.explode(columns[0]), validate_column_names=False
             )
-        else:
-            native_frame = self.native
-            anchor_series = native_frame[columns[0]].list.len()
+        native_frame = self.native
+        anchor_series = native_frame[columns[0]].list.len()
 
-            if not all(
-                (native_frame[col_name].list.len() == anchor_series).all()
-                for col_name in columns[1:]
-            ):
-                msg = "exploded columns must have matching element counts"
-                raise ShapeError(msg)
+        if not all(
+            (native_frame[col_name].list.len() == anchor_series).all()
+            for col_name in columns[1:]
+        ):
+            msg = "exploded columns must have matching element counts"
+            raise ShapeError(msg)
 
-            original_columns = self.columns
-            other_columns = [c for c in original_columns if c not in columns]
+        original_columns = self.columns
+        other_columns = [c for c in original_columns if c not in columns]
 
-            exploded_frame = native_frame[[*other_columns, columns[0]]].explode(
-                columns[0]
-            )
-            exploded_series = [
-                native_frame[col_name].explode().to_frame() for col_name in columns[1:]
-            ]
+        exploded_frame = native_frame[[*other_columns, columns[0]]].explode(columns[0])
+        exploded_series = [
+            native_frame[col_name].explode().to_frame() for col_name in columns[1:]
+        ]
 
-            plx = self.__native_namespace__()
-            return self._with_native(
-                plx.concat([exploded_frame, *exploded_series], axis=1)[original_columns],
-                validate_column_names=False,
-            )
+        plx = self.__native_namespace__()
+        return self._with_native(
+            plx.concat([exploded_frame, *exploded_series], axis=1)[original_columns],
+            validate_column_names=False,
+        )
