@@ -4,18 +4,15 @@ import platform
 import sys
 from collections.abc import Iterable, Mapping, Sequence
 from functools import partial
-from importlib.metadata import version
-from typing import TYPE_CHECKING, Any, Literal, cast
+from typing import TYPE_CHECKING, Any, Callable
 
 from narwhals._expression_parsing import (
     ExprKind,
     ExprMetadata,
     apply_n_ary_operation,
     combine_metadata,
-    extract_compliant,
     is_scalar_like,
 )
-from narwhals._typing_compat import deprecated
 from narwhals._utils import (
     Implementation,
     Version,
@@ -24,7 +21,7 @@ from narwhals._utils import (
     is_compliant_expr,
     is_eager_allowed,
     is_sequence_but_not_str,
-    issue_deprecation_warning,
+    normalize_path,
     supports_arrow_c_stream,
     validate_laziness,
 )
@@ -46,16 +43,16 @@ if TYPE_CHECKING:
 
     from narwhals._compliant import CompliantExpr, CompliantNamespace
     from narwhals._translate import IntoArrowTable
+    from narwhals._typing import Backend, EagerAllowed, IntoBackend
     from narwhals.dataframe import DataFrame, LazyFrame
-    from narwhals.dtypes import DType
-    from narwhals.schema import Schema
     from narwhals.typing import (
         ConcatMethod,
+        FileSource,
         FrameT,
         IntoDType,
         IntoExpr,
-        IntoSeriesT,
-        NativeFrame,
+        IntoSchema,
+        NativeDataFrame,
         NativeLazyFrame,
         NativeSeries,
         NonNestedLiteral,
@@ -63,7 +60,7 @@ if TYPE_CHECKING:
         _2DArray,
     )
 
-    _IntoSchema: TypeAlias = "Mapping[str, DType] | Schema | Sequence[str] | None"
+    _IntoSchema: TypeAlias = "IntoSchema | Sequence[str] | None"
 
 
 def concat(items: Iterable[FrameT], *, how: ConcatMethod = "vertical") -> FrameT:
@@ -171,14 +168,12 @@ def concat(items: Iterable[FrameT], *, how: ConcatMethod = "vertical") -> FrameT
     )
 
 
-@deprecate_native_namespace(warn_version="1.31.0", required=True)
 def new_series(
     name: str,
     values: Any,
     dtype: IntoDType | None = None,
     *,
-    backend: ModuleType | Implementation | str | None = None,
-    native_namespace: ModuleType | None = None,  # noqa: ARG001
+    backend: IntoBackend[EagerAllowed],
 ) -> Series[Any]:
     """Instantiate Narwhals Series from iterable (e.g. list or array).
 
@@ -195,13 +190,6 @@ def new_series(
                 `POLARS`, `MODIN` or `CUDF`.
             - As a string: `"pandas"`, `"pyarrow"`, `"polars"`, `"modin"` or `"cudf"`.
             - Directly as a module `pandas`, `pyarrow`, `polars`, `modin` or `cudf`.
-        native_namespace: The native library to use for DataFrame creation.
-
-            *Deprecated* (v1.31.0)
-
-            Please use `backend` instead. Note that `native_namespace` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
 
     Returns:
         A new Series
@@ -222,7 +210,6 @@ def new_series(
         |Name: a, dtype: int32|
         └─────────────────────┘
     """
-    backend = cast("ModuleType | Implementation | str", backend)
     return _new_series_impl(name, values, dtype, backend=backend)
 
 
@@ -231,14 +218,14 @@ def _new_series_impl(
     values: Any,
     dtype: IntoDType | None = None,
     *,
-    backend: ModuleType | Implementation | str,
+    backend: IntoBackend[EagerAllowed],
 ) -> Series[Any]:
     implementation = Implementation.from_backend(backend)
     if is_eager_allowed(implementation):
         ns = Version.MAIN.namespace.from_backend(implementation).compliant
         series = ns._series.from_iterable(values, name=name, context=ns, dtype=dtype)
         return series.to_narwhals()
-    elif implementation is Implementation.UNKNOWN:  # pragma: no cover
+    if implementation is Implementation.UNKNOWN:  # pragma: no cover
         _native_namespace = implementation.to_native_namespace()
         try:
             native_series: NativeSeries = _native_namespace.new_series(
@@ -259,9 +246,9 @@ def _new_series_impl(
 @deprecate_native_namespace(warn_version="1.26.0")
 def from_dict(
     data: Mapping[str, Any],
-    schema: Mapping[str, DType] | Schema | None = None,
+    schema: IntoSchema | None = None,
     *,
-    backend: ModuleType | Implementation | str | None = None,
+    backend: IntoBackend[EagerAllowed] | None = None,
     native_namespace: ModuleType | None = None,  # noqa: ARG001
 ) -> DataFrame[Any]:
     """Instantiate DataFrame from dictionary.
@@ -286,13 +273,7 @@ def from_dict(
                 `POLARS`, `MODIN` or `CUDF`.
             - As a string: `"pandas"`, `"pyarrow"`, `"polars"`, `"modin"` or `"cudf"`.
             - Directly as a module `pandas`, `pyarrow`, `polars`, `modin` or `cudf`.
-        native_namespace: The native library to use for DataFrame creation.
-
-            *Deprecated* (v1.26.0)
-
-            Please use `backend` instead. Note that `native_namespace` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
+        native_namespace: deprecated, same as `backend`.
 
     Returns:
         A new DataFrame.
@@ -310,21 +291,20 @@ def from_dict(
         |     1  2  4      |
         └──────────────────┘
     """
-    if not data:
-        msg = "from_dict cannot be called with empty dictionary"
-        raise ValueError(msg)
     if backend is None:
         data, backend = _from_dict_no_backend(data)
     implementation = Implementation.from_backend(backend)
     if is_eager_allowed(implementation):
         ns = Version.MAIN.namespace.from_backend(implementation).compliant
         return ns._dataframe.from_dict(data, schema=schema, context=ns).to_narwhals()
-    elif implementation is Implementation.UNKNOWN:  # pragma: no cover
+    if implementation is Implementation.UNKNOWN:  # pragma: no cover
         _native_namespace = implementation.to_native_namespace()
         try:
             # implementation is UNKNOWN, Narwhals extension using this feature should
             # implement `from_dict` function in the top-level namespace.
-            native_frame: NativeFrame = _native_namespace.from_dict(data, schema=schema)
+            native_frame: NativeDataFrame = _native_namespace.from_dict(
+                data, schema=schema
+            )
         except AttributeError as e:
             msg = "Unknown namespace is expected to implement `from_dict` function."
             raise AttributeError(msg) from e
@@ -351,13 +331,11 @@ def _from_dict_no_backend(
     return data, native_namespace
 
 
-@deprecate_native_namespace(warn_version="1.31.0", required=True)
 def from_numpy(
     data: _2DArray,
-    schema: Mapping[str, DType] | Schema | Sequence[str] | None = None,
+    schema: IntoSchema | Sequence[str] | None = None,
     *,
-    backend: ModuleType | Implementation | str | None = None,
-    native_namespace: ModuleType | None = None,  # noqa: ARG001
+    backend: IntoBackend[EagerAllowed],
 ) -> DataFrame[Any]:
     """Construct a DataFrame from a NumPy ndarray.
 
@@ -378,13 +356,6 @@ def from_numpy(
                 `POLARS`, `MODIN` or `CUDF`.
             - As a string: `"pandas"`, `"pyarrow"`, `"polars"`, `"modin"` or `"cudf"`.
             - Directly as a module `pandas`, `pyarrow`, `polars`, `modin` or `cudf`.
-        native_namespace: The native library to use for DataFrame creation.
-
-            *Deprecated* (v1.31.0)
-
-            Please use `backend` instead. Note that `native_namespace` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
 
     Returns:
         A new DataFrame.
@@ -410,14 +381,13 @@ def from_numpy(
         |  e: [[1,3]]      |
         └──────────────────┘
     """
-    backend = cast("ModuleType | Implementation | str", backend)
     if not is_numpy_array_2d(data):
         msg = "`from_numpy` only accepts 2D numpy arrays"
         raise ValueError(msg)
     if not _is_into_schema(schema):
         msg = (
             "`schema` is expected to be one of the following types: "
-            "Mapping[str, DType] | Schema | Sequence[str]. "
+            "IntoSchema | Sequence[str]. "
             f"Got {type(schema)}."
         )
         raise TypeError(msg)
@@ -425,12 +395,14 @@ def from_numpy(
     if is_eager_allowed(implementation):
         ns = Version.MAIN.namespace.from_backend(implementation).compliant
         return ns.from_numpy(data, schema).to_narwhals()
-    elif implementation is Implementation.UNKNOWN:  # pragma: no cover
+    if implementation is Implementation.UNKNOWN:  # pragma: no cover
         _native_namespace = implementation.to_native_namespace()
         try:
             # implementation is UNKNOWN, Narwhals extension using this feature should
             # implement `from_numpy` function in the top-level namespace.
-            native_frame: NativeFrame = _native_namespace.from_numpy(data, schema=schema)
+            native_frame: NativeDataFrame = _native_namespace.from_numpy(
+                data, schema=schema
+            )
         except AttributeError as e:
             msg = "Unknown namespace is expected to implement `from_numpy` function."
             raise AttributeError(msg) from e
@@ -451,12 +423,8 @@ def _is_into_schema(obj: Any) -> TypeIs[_IntoSchema]:
     )
 
 
-@deprecate_native_namespace(warn_version="1.31.0", required=True)
 def from_arrow(
-    native_frame: IntoArrowTable,
-    *,
-    backend: ModuleType | Implementation | str | None = None,
-    native_namespace: ModuleType | None = None,  # noqa: ARG001
+    native_frame: IntoArrowTable, *, backend: IntoBackend[EagerAllowed]
 ) -> DataFrame[Any]:  # pragma: no cover
     """Construct a DataFrame from an object which supports the PyCapsule Interface.
 
@@ -470,13 +438,6 @@ def from_arrow(
                 `POLARS`, `MODIN` or `CUDF`.
             - As a string: `"pandas"`, `"pyarrow"`, `"polars"`, `"modin"` or `"cudf"`.
             - Directly as a module `pandas`, `pyarrow`, `polars`, `modin` or `cudf`.
-        native_namespace: The native library to use for DataFrame creation.
-
-            *Deprecated* (v1.31.0)
-
-            Please use `backend` instead. Note that `native_namespace` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
 
     Returns:
         A new DataFrame.
@@ -502,7 +463,6 @@ def from_arrow(
         |  └─────┴─────┘   |
         └──────────────────┘
     """
-    backend = cast("ModuleType | Implementation | str", backend)
     if not (supports_arrow_c_stream(native_frame) or is_pyarrow_table(native_frame)):
         msg = f"Given object of type {type(native_frame)} does not support PyCapsule interface"
         raise TypeError(msg)
@@ -510,12 +470,12 @@ def from_arrow(
     if is_eager_allowed(implementation):
         ns = Version.MAIN.namespace.from_backend(implementation).compliant
         return ns._dataframe.from_arrow(native_frame, context=ns).to_narwhals()
-    elif implementation is Implementation.UNKNOWN:  # pragma: no cover
+    if implementation is Implementation.UNKNOWN:  # pragma: no cover
         _native_namespace = implementation.to_native_namespace()
         try:
             # implementation is UNKNOWN, Narwhals extension using this feature should
             # implement PyCapsule support
-            native: NativeFrame = _native_namespace.DataFrame(native_frame)
+            native: NativeDataFrame = _native_namespace.DataFrame(native_frame)
         except AttributeError as e:
             msg = "Unknown namespace is expected to implement `DataFrame` class which accepts object which supports PyCapsule Interface."
             raise AttributeError(msg) from e
@@ -562,19 +522,28 @@ def _get_deps_info() -> dict[str, str]:
     Returns:
         Mapping from dependency to version.
     """
-    from importlib.metadata import PackageNotFoundError
+    from importlib.metadata import distributions
 
-    from narwhals import __version__
+    extra_names = ("narwhals", "numpy")
+    member_names = Implementation._member_names_
+    exclude = {"PYSPARK_CONNECT", "UNKNOWN"}
+    target_names = tuple(
+        name.lower() for name in (*extra_names, *member_names) if name not in exclude
+    )
+    result = dict.fromkeys(target_names, "")  # Initialize with empty strings
 
-    deps = ("pandas", "polars", "cudf", "modin", "pyarrow", "numpy")
-    deps_info = {"narwhals": __version__}
+    for dist in distributions():
+        dist_name, dist_version = dist.name.lower(), dist.version
 
-    for modname in deps:
-        try:
-            deps_info[modname] = version(modname)
-        except PackageNotFoundError:  # noqa: PERF203
-            deps_info[modname] = ""
-    return deps_info
+        if dist_name in result:  # exact match
+            result[dist_name] = dist_version
+        else:  # prefix match
+            for target in target_names:
+                if not result[target] and dist_name.startswith(target):
+                    result[target] = dist_version
+                    break
+
+    return result
 
 
 def show_versions() -> None:
@@ -596,47 +565,8 @@ def show_versions() -> None:
         print(f"{k:>13}: {stat}")  # noqa: T201
 
 
-@deprecated(
-    "`get_level` is deprecated, as Narwhals no longer supports the Dataframe Interchange Protocol."
-)
-def get_level(
-    obj: DataFrame[Any] | LazyFrame[Any] | Series[IntoSeriesT],
-) -> Literal["full", "lazy", "interchange"]:
-    """Level of support Narwhals has for current object.
-
-    Warning:
-        `get_level` is deprecated and will be removed in a future version.
-        "DuckDB and Ibis now have full lazy support in Narwhals, and passing
-        them to `nw.from_native` returns `nw.LazyFrame`.
-        Note: this will remain available in `narwhals.stable.v1`.
-        See [stable api](../backcompat.md/) for more information.
-
-    Arguments:
-        obj: Dataframe or Series.
-
-    Returns:
-        This can be one of
-
-            - 'full': full Narwhals API support
-            - 'lazy': only lazy operations are supported. This excludes anything
-              which involves iterating over rows in Python.
-    """
-    issue_deprecation_warning(
-        "`get_level` is deprecated, as Narwhals no longer supports the Dataframe Interchange Protocol.\n"
-        "DuckDB and Ibis now have full lazy support in Narwhals, and passing them to `nw.from_native` \n"
-        "returns `nw.LazyFrame`.",
-        "1.43",
-    )
-    return obj._level
-
-
-@deprecate_native_namespace(warn_version="1.27.2", required=True)
 def read_csv(
-    source: str,
-    *,
-    backend: ModuleType | Implementation | str | None = None,
-    native_namespace: ModuleType | None = None,
-    **kwargs: Any,
+    source: FileSource, *, backend: IntoBackend[EagerAllowed], **kwargs: Any
 ) -> DataFrame[Any]:
     """Read a CSV file into a DataFrame.
 
@@ -649,13 +579,6 @@ def read_csv(
                 `POLARS`, `MODIN` or `CUDF`.
             - As a string: `"pandas"`, `"pyarrow"`, `"polars"`, `"modin"` or `"cudf"`.
             - Directly as a module `pandas`, `pyarrow`, `polars`, `modin` or `cudf`.
-        native_namespace: The native library to use for DataFrame creation.
-
-            *Deprecated* (v1.27.2)
-
-            Please use `backend` instead. Note that `native_namespace` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
         kwargs: Extra keyword arguments which are passed to the native CSV reader.
             For example, you could use
             `nw.read_csv('file.csv', backend='pandas', engine='pyarrow')`.
@@ -674,21 +597,33 @@ def read_csv(
         |     1  2   5     |
         └──────────────────┘
     """
-    backend = cast("ModuleType | Implementation | str", backend)
-    eager_backend = Implementation.from_backend(backend)
-    native_namespace = eager_backend.to_native_namespace()
-    native_frame: NativeFrame
-    if eager_backend in {
+    impl = Implementation.from_backend(backend)
+    native_namespace = impl.to_native_namespace()
+    native_frame: NativeDataFrame
+    if impl in {
         Implementation.POLARS,
         Implementation.PANDAS,
         Implementation.MODIN,
         Implementation.CUDF,
     }:
-        native_frame = native_namespace.read_csv(source, **kwargs)
-    elif eager_backend is Implementation.PYARROW:
+        native_frame = native_namespace.read_csv(normalize_path(source), **kwargs)
+    elif impl is Implementation.PYARROW:
         from pyarrow import csv  # ignore-banned-import
 
         native_frame = csv.read_csv(source, **kwargs)
+    elif impl in {
+        Implementation.PYSPARK,
+        Implementation.DASK,
+        Implementation.DUCKDB,
+        Implementation.IBIS,
+        Implementation.SQLFRAME,
+        Implementation.PYSPARK_CONNECT,
+    }:
+        msg = (
+            f"Expected eager backend, found {impl}.\n\n"
+            f"Hint: use nw.scan_csv(source={source}, backend={backend})"
+        )
+        raise ValueError(msg)
     else:  # pragma: no cover
         try:
             # implementation is UNKNOWN, Narwhals extension using this feature should
@@ -700,13 +635,8 @@ def read_csv(
     return from_native(native_frame, eager_only=True)
 
 
-@deprecate_native_namespace(warn_version="1.31.0", required=True)
 def scan_csv(
-    source: str,
-    *,
-    backend: ModuleType | Implementation | str | None = None,
-    native_namespace: ModuleType | None = None,
-    **kwargs: Any,
+    source: FileSource, *, backend: IntoBackend[Backend], **kwargs: Any
 ) -> LazyFrame[Any]:
     """Lazily read from a CSV file.
 
@@ -722,13 +652,6 @@ def scan_csv(
                 `POLARS`, `MODIN` or `CUDF`.
             - As a string: `"pandas"`, `"pyarrow"`, `"polars"`, `"modin"` or `"cudf"`.
             - Directly as a module `pandas`, `pyarrow`, `polars`, `modin` or `cudf`.
-        native_namespace: The native library to use for DataFrame creation.
-
-            *Deprecated* (v1.31.0)
-
-            Please use `backend` instead. Note that `native_namespace` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
         kwargs: Extra keyword arguments which are passed to the native CSV reader.
             For example, you could use
             `nw.scan_csv('file.csv', backend=pd, engine='pyarrow')`.
@@ -750,10 +673,10 @@ def scan_csv(
         │ z       │     3 │
         └─────────┴───────┘
     """
-    backend = cast("ModuleType | Implementation | str", backend)
     implementation = Implementation.from_backend(backend)
     native_namespace = implementation.to_native_namespace()
-    native_frame: NativeFrame | NativeLazyFrame
+    native_frame: NativeDataFrame | NativeLazyFrame
+    source = normalize_path(source)
     if implementation is Implementation.POLARS:
         native_frame = native_namespace.scan_csv(source, **kwargs)
     elif implementation in {
@@ -773,7 +696,6 @@ def scan_csv(
         if (session := kwargs.pop("session", None)) is None:
             msg = "Spark like backends require a session object to be passed in `kwargs`."
             raise ValueError(msg)
-
         csv_reader = session.read.format("csv")
         native_frame = (
             csv_reader.load(source)
@@ -794,13 +716,8 @@ def scan_csv(
     return from_native(native_frame).lazy()
 
 
-@deprecate_native_namespace(warn_version="1.31.0", required=True)
 def read_parquet(
-    source: str,
-    *,
-    backend: ModuleType | Implementation | str | None = None,
-    native_namespace: ModuleType | None = None,
-    **kwargs: Any,
+    source: FileSource, *, backend: IntoBackend[EagerAllowed], **kwargs: Any
 ) -> DataFrame[Any]:
     """Read into a DataFrame from a parquet file.
 
@@ -813,13 +730,6 @@ def read_parquet(
                 `POLARS`, `MODIN` or `CUDF`.
             - As a string: `"pandas"`, `"pyarrow"`, `"polars"`, `"modin"` or `"cudf"`.
             - Directly as a module `pandas`, `pyarrow`, `polars`, `modin` or `cudf`.
-        native_namespace: The native library to use for DataFrame creation.
-
-            *Deprecated* (v1.31.0)
-
-            Please use `backend` instead. Note that `native_namespace` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
         kwargs: Extra keyword arguments which are passed to the native parquet reader.
             For example, you could use
             `nw.read_parquet('file.parquet', backend=pd, engine='pyarrow')`.
@@ -843,23 +753,34 @@ def read_parquet(
         |c: [[0.2,0.1]]    |
         └──────────────────┘
     """
-    backend = cast("ModuleType | Implementation | str", backend)
-    implementation = Implementation.from_backend(backend)
-    native_namespace = implementation.to_native_namespace()
-    native_frame: NativeFrame
-    if implementation in {
+    impl = Implementation.from_backend(backend)
+    native_namespace = impl.to_native_namespace()
+    native_frame: NativeDataFrame
+    if impl in {
         Implementation.POLARS,
         Implementation.PANDAS,
         Implementation.MODIN,
         Implementation.CUDF,
-        Implementation.DUCKDB,
-        Implementation.IBIS,
     }:
+        source = normalize_path(source)
         native_frame = native_namespace.read_parquet(source, **kwargs)
-    elif implementation is Implementation.PYARROW:
+    elif impl is Implementation.PYARROW:
         import pyarrow.parquet as pq  # ignore-banned-import
 
-        native_frame = pq.read_table(source, **kwargs)
+        native_frame = pq.read_table(source, **kwargs)  # type: ignore[arg-type]
+    elif impl in {
+        Implementation.PYSPARK,
+        Implementation.DASK,
+        Implementation.DUCKDB,
+        Implementation.IBIS,
+        Implementation.SQLFRAME,
+        Implementation.PYSPARK_CONNECT,
+    }:
+        msg = (
+            f"Expected eager backend, found {impl}.\n\n"
+            f"Hint: use nw.scan_parquet(source={source}, backend={backend})"
+        )
+        raise ValueError(msg)
     else:  # pragma: no cover
         try:
             # implementation is UNKNOWN, Narwhals extension using this feature should
@@ -871,13 +792,8 @@ def read_parquet(
     return from_native(native_frame, eager_only=True)
 
 
-@deprecate_native_namespace(warn_version="1.31.0", required=True)
 def scan_parquet(
-    source: str,
-    *,
-    backend: ModuleType | Implementation | str | None = None,
-    native_namespace: ModuleType | None = None,
-    **kwargs: Any,
+    source: FileSource, *, backend: IntoBackend[Backend], **kwargs: Any
 ) -> LazyFrame[Any]:
     """Lazily read from a parquet file.
 
@@ -907,13 +823,6 @@ def scan_parquet(
                 `"pyspark"` or `"sqlframe"`.
             - Directly as a module `pandas`, `pyarrow`, `polars`, `modin`, `cudf`,
                 `pyspark.sql` or `sqlframe`.
-        native_namespace: The native library to use for DataFrame creation.
-
-            *Deprecated* (v1.31.0)
-
-            Please use `backend` instead. Note that `native_namespace` is still available
-            (and won't emit a deprecation warning) if you use `narwhals.stable.v1`,
-            see [perfect backwards compatibility policy](../backcompat.md/).
         kwargs: Extra keyword arguments which are passed to the native parquet reader.
             For example, you could use
             `nw.scan_parquet('file.parquet', backend=pd, engine='pyarrow')`.
@@ -948,10 +857,10 @@ def scan_parquet(
         |  b: [[4,5]]      |
         └──────────────────┘
     """
-    backend = cast("ModuleType | Implementation | str", backend)
     implementation = Implementation.from_backend(backend)
     native_namespace = implementation.to_native_namespace()
-    native_frame: NativeFrame | NativeLazyFrame
+    native_frame: NativeDataFrame | NativeLazyFrame
+    source = normalize_path(source)
     if implementation is Implementation.POLARS:
         native_frame = native_namespace.scan_parquet(source, **kwargs)
     elif implementation in {
@@ -971,7 +880,6 @@ def scan_parquet(
         if (session := kwargs.pop("session", None)) is None:
             msg = "Spark like backends require a session object to be passed in `kwargs`."
             raise ValueError(msg)
-
         pq_reader = session.read.format("parquet")
         native_frame = (
             pq_reader.load(source)
@@ -1328,6 +1236,24 @@ def max(*columns: str) -> Expr:
     return col(*columns).max()
 
 
+def _expr_with_n_ary_op(
+    func_name: str,
+    operation_factory: Callable[
+        [CompliantNamespace[Any, Any]], Callable[..., CompliantExpr[Any, Any]]
+    ],
+    *exprs: IntoExpr,
+) -> Expr:
+    if not exprs:
+        msg = f"At least one expression must be passed to `{func_name}`"
+        raise ValueError(msg)
+    return Expr(
+        lambda plx: apply_n_ary_operation(
+            plx, operation_factory(plx), *exprs, str_as_lit=False
+        ),
+        ExprMetadata.from_horizontal_op(*exprs),
+    )
+
+
 def sum_horizontal(*exprs: IntoExpr | Iterable[IntoExpr]) -> Expr:
     """Sum all values horizontally across columns.
 
@@ -1362,15 +1288,8 @@ def sum_horizontal(*exprs: IntoExpr | Iterable[IntoExpr]) -> Expr:
         |└─────┴──────┴─────┘|
         └────────────────────┘
     """
-    if not exprs:
-        msg = "At least one expression must be passed to `sum_horizontal`"
-        raise ValueError(msg)
-    flat_exprs = flatten(exprs)
-    return Expr(
-        lambda plx: apply_n_ary_operation(
-            plx, plx.sum_horizontal, *flat_exprs, str_as_lit=False
-        ),
-        ExprMetadata.from_horizontal_op(*flat_exprs),
+    return _expr_with_n_ary_op(
+        "sum_horizontal", lambda plx: plx.sum_horizontal, *flatten(exprs)
     )
 
 
@@ -1406,15 +1325,8 @@ def min_horizontal(*exprs: IntoExpr | Iterable[IntoExpr]) -> Expr:
         | h_min: [[1,5,3]] |
         └──────────────────┘
     """
-    if not exprs:
-        msg = "At least one expression must be passed to `min_horizontal`"
-        raise ValueError(msg)
-    flat_exprs = flatten(exprs)
-    return Expr(
-        lambda plx: apply_n_ary_operation(
-            plx, plx.min_horizontal, *flat_exprs, str_as_lit=False
-        ),
-        ExprMetadata.from_horizontal_op(*flat_exprs),
+    return _expr_with_n_ary_op(
+        "min_horizontal", lambda plx: plx.min_horizontal, *flatten(exprs)
     )
 
 
@@ -1452,15 +1364,8 @@ def max_horizontal(*exprs: IntoExpr | Iterable[IntoExpr]) -> Expr:
         |└─────┴──────┴───────┘|
         └──────────────────────┘
     """
-    if not exprs:
-        msg = "At least one expression must be passed to `max_horizontal`"
-        raise ValueError(msg)
-    flat_exprs = flatten(exprs)
-    return Expr(
-        lambda plx: apply_n_ary_operation(
-            plx, plx.max_horizontal, *flat_exprs, str_as_lit=False
-        ),
-        ExprMetadata.from_horizontal_op(*flat_exprs),
+    return _expr_with_n_ary_op(
+        "max_horizontal", lambda plx: plx.max_horizontal, *flatten(exprs)
     )
 
 
@@ -1507,7 +1412,7 @@ class Then(Expr):
 
         def func(plx: CompliantNamespace[Any, Any]) -> CompliantExpr[Any, Any]:
             compliant_expr = self._to_compliant_expr(plx)
-            compliant_value = extract_compliant(plx, value, str_as_lit=False)
+            compliant_value = plx.parse_into_expr(value, str_as_lit=False)
             if (
                 not self._metadata.is_scalar_like
                 and is_scalar_like(kind)
@@ -1570,9 +1475,7 @@ def when(*predicates: IntoExpr | Iterable[IntoExpr]) -> When:
     return When(*predicates)
 
 
-def all_horizontal(
-    *exprs: IntoExpr | Iterable[IntoExpr], ignore_nulls: bool | None = None
-) -> Expr:
+def all_horizontal(*exprs: IntoExpr | Iterable[IntoExpr], ignore_nulls: bool) -> Expr:
     r"""Compute the bitwise AND horizontally across columns.
 
     Arguments:
@@ -1582,7 +1485,7 @@ def all_horizontal(
 
             - If `True`, null values are ignored. If there are no elements, the result
               is `True`.
-            - If `False` (default), Kleene logic is followed. Note that this is not allowed for
+            - If `False`, Kleene logic is followed. Note that this is not allowed for
               pandas with classical NumPy dtypes when null values are present.
 
     Returns:
@@ -1614,24 +1517,10 @@ def all_horizontal(
         └─────────────────────────────────────────┘
 
     """
-    if not exprs:
-        msg = "At least one expression must be passed to `all_horizontal`"
-        raise ValueError(msg)
-    if ignore_nulls is None:
-        issue_deprecation_warning(
-            "`ignore_nulls` will become a required argument in Narwhals 2.0. Please specify `ignore_nulls=True` or `ignore_nulls=False` to silence this warning.",
-            _version="1.45",
-        )
-        ignore_nulls = False
-    flat_exprs = flatten(exprs)
-    return Expr(
-        lambda plx: apply_n_ary_operation(
-            plx,
-            partial(plx.all_horizontal, ignore_nulls=ignore_nulls),
-            *flat_exprs,
-            str_as_lit=False,
-        ),
-        ExprMetadata.from_horizontal_op(*flat_exprs),
+    return _expr_with_n_ary_op(
+        "all_horizontal",
+        lambda plx: partial(plx.all_horizontal, ignore_nulls=ignore_nulls),
+        *flatten(exprs),
     )
 
 
@@ -1674,9 +1563,7 @@ def lit(value: NonNestedLiteral, dtype: IntoDType | None = None) -> Expr:
     return Expr(lambda plx: plx.lit(value, dtype), ExprMetadata.literal())
 
 
-def any_horizontal(
-    *exprs: IntoExpr | Iterable[IntoExpr], ignore_nulls: bool | None = None
-) -> Expr:
+def any_horizontal(*exprs: IntoExpr | Iterable[IntoExpr], ignore_nulls: bool) -> Expr:
     r"""Compute the bitwise OR horizontally across columns.
 
     Arguments:
@@ -1686,7 +1573,7 @@ def any_horizontal(
 
             - If `True`, null values are ignored. If there are no elements, the result
               is `False`.
-            - If `False` (default), Kleene logic is followed. Note that this is not allowed for
+            - If `False`, Kleene logic is followed. Note that this is not allowed for
               pandas with classical NumPy dtypes when null values are present.
 
     Returns:
@@ -1722,24 +1609,10 @@ def any_horizontal(
         |└───────┴───────┴───────┘|
         └─────────────────────────┘
     """
-    if not exprs:
-        msg = "At least one expression must be passed to `any_horizontal`"
-        raise ValueError(msg)
-    if ignore_nulls is None:
-        issue_deprecation_warning(
-            "`ignore_nulls` will become a required argument in Narwhals 2.0. Please specify `ignore_nulls=True` or `ignore_nulls=False` to silence this warning.",
-            _version="1.45",
-        )
-        ignore_nulls = False
-    flat_exprs = flatten(exprs)
-    return Expr(
-        lambda plx: apply_n_ary_operation(
-            plx,
-            partial(plx.any_horizontal, ignore_nulls=ignore_nulls),
-            *flat_exprs,
-            str_as_lit=False,
-        ),
-        ExprMetadata.from_horizontal_op(*flat_exprs),
+    return _expr_with_n_ary_op(
+        "any_horizontal",
+        lambda plx: partial(plx.any_horizontal, ignore_nulls=ignore_nulls),
+        *flatten(exprs),
     )
 
 
@@ -1773,15 +1646,8 @@ def mean_horizontal(*exprs: IntoExpr | Iterable[IntoExpr]) -> Expr:
         | a: [[2.5,6.5,3]] |
         └──────────────────┘
     """
-    if not exprs:
-        msg = "At least one expression must be passed to `mean_horizontal`"
-        raise ValueError(msg)
-    flat_exprs = flatten(exprs)
-    return Expr(
-        lambda plx: apply_n_ary_operation(
-            plx, plx.mean_horizontal, *flat_exprs, str_as_lit=False
-        ),
-        ExprMetadata.from_horizontal_op(*flat_exprs),
+    return _expr_with_n_ary_op(
+        "mean_horizontal", lambda plx: plx.mean_horizontal, *flatten(exprs)
     )
 
 
@@ -1834,18 +1700,12 @@ def concat_str(
         └──────────────────┘
     """
     flat_exprs = flatten([*flatten([exprs]), *more_exprs])
-    return Expr(
-        lambda plx: apply_n_ary_operation(
-            plx,
-            lambda *args: plx.concat_str(
-                *args, separator=separator, ignore_nulls=ignore_nulls
-            ),
-            *flat_exprs,
-            str_as_lit=False,
-        ),
-        combine_metadata(
-            *flat_exprs, str_as_lit=False, allow_multi_output=True, to_single_output=True
+    return _expr_with_n_ary_op(
+        "concat_str",
+        lambda plx: lambda *args: plx.concat_str(
+            *args, separator=separator, ignore_nulls=ignore_nulls
         ),
+        *flat_exprs,
     )
 
 
@@ -1868,33 +1728,33 @@ def coalesce(
         A new expression.
 
     Examples:
-    >>> import polars as pl
-    >>> import narwhals as nw
-    >>> data = [
-    ...     (1, 5, None),
-    ...     (None, 6, None),
-    ...     (None, None, 9),
-    ...     (4, 8, 10),
-    ...     (None, None, None),
-    ... ]
-    >>> df = pl.DataFrame(data, schema=["a", "b", "c"], orient="row")
-    >>> nw.from_native(df).select(nw.coalesce("a", "b", "c", nw.lit(-1)))
-    ┌──────────────────┐
-    |Narwhals DataFrame|
-    |------------------|
-    |  shape: (5, 1)   |
-    |  ┌─────┐         |
-    |  │ a   │         |
-    |  │ --- │         |
-    |  │ i64 │         |
-    |  ╞═════╡         |
-    |  │ 1   │         |
-    |  │ 6   │         |
-    |  │ 9   │         |
-    |  │ 4   │         |
-    |  │ -1  │         |
-    |  └─────┘         |
-    └──────────────────┘
+        >>> import polars as pl
+        >>> import narwhals as nw
+        >>> data = [
+        ...     (1, 5, None),
+        ...     (None, 6, None),
+        ...     (None, None, 9),
+        ...     (4, 8, 10),
+        ...     (None, None, None),
+        ... ]
+        >>> df = pl.DataFrame(data, schema=["a", "b", "c"], orient="row")
+        >>> nw.from_native(df).select(nw.coalesce("a", "b", "c", nw.lit(-1)))
+        ┌──────────────────┐
+        |Narwhals DataFrame|
+        |------------------|
+        |  shape: (5, 1)   |
+        |  ┌─────┐         |
+        |  │ a   │         |
+        |  │ --- │         |
+        |  │ i64 │         |
+        |  ╞═════╡         |
+        |  │ 1   │         |
+        |  │ 6   │         |
+        |  │ 9   │         |
+        |  │ 4   │         |
+        |  │ -1  │         |
+        |  └─────┘         |
+        └──────────────────┘
     """
     flat_exprs = flatten([*flatten([exprs]), *more_exprs])
 
