@@ -1,7 +1,9 @@
 from __future__ import annotations
 
+import pytest
+
 import narwhals as nw
-from tests.utils import Constructor, assert_equal_data
+from tests.utils import POLARS_VERSION, Constructor, assert_equal_data
 
 data = {"foo": [1, 2, 3], "BAR": [4, 5, 6]}
 suffix = "_with_suffix"
@@ -15,9 +17,16 @@ def test_suffix(constructor: Constructor) -> None:
 
 
 def test_suffix_after_alias(constructor: Constructor) -> None:
+    if "polars" in str(constructor) and POLARS_VERSION < (1, 32):
+        pytest.skip(reason="https://github.com/pola-rs/polars/issues/23765")
     df = nw.from_native(constructor(data))
     result = df.select((nw.col("foo")).alias("alias_for_foo").name.suffix(suffix))
-    expected = {"foo_with_suffix": [1, 2, 3]}
+    expected = {"alias_for_foo_with_suffix": [1, 2, 3]}
+    assert_equal_data(result, expected)
+    result = df.select(
+        nw.col("foo").alias("bar").name.prefix("prefix_").name.suffix("_suffix")
+    )
+    expected = {"prefix_bar_suffix": [1, 2, 3]}
     assert_equal_data(result, expected)
 
 
