@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+from typing import cast
+
 import pandas as pd
 import pyarrow as pa
 import pytest
@@ -18,7 +20,7 @@ def test_get_field_expr(request: pytest.FixtureRequest, constructor: Constructor
     df_native = constructor(data)
 
     if "pandas" in str(constructor):
-        df_native = df_native.assign(  # type: ignore[union-attr]
+        df_native = cast("pd.DataFrame", df_native).assign(
             user=pd.Series(
                 data["user"],
                 dtype=pd.ArrowDtype(
@@ -53,7 +55,7 @@ def test_get_field_series(
     df_native = constructor_eager(data)
 
     if "pandas" in str(constructor_eager):
-        df_native = df_native.assign(  # type: ignore[union-attr]
+        df_native = cast("pd.DataFrame", df_native).assign(
             user=pd.Series(
                 data["user"],
                 dtype=pd.ArrowDtype(
