@@ -6,10 +6,7 @@ import narwhals as nw
 
 if TYPE_CHECKING:
     from tests.utils import ConstructorEager
-data = {
-    "a": [1.0, 2.0, None, 4.0],
-    "b": [None, 3.0, None, 5.0],
-}
+data = {"a": [1.0, 2.0, None, 4.0], "b": [None, 3.0, None, 5.0]}
 
 
 def test_len(constructor_eager: ConstructorEager) -> None:
